
/* -*- tab-width: 8; c-basic-offset: 4; indent-tabs-mode: nil; -*- */
/*
 * Copyright (C) 2012 Canonical, Ltd.
 *
 * Authors:
 *  Renato Araujo Oliveira Filho <renato@canonical.com>
 *  Conor Curran <conor.curran@canonical.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; version 3.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
 
using Dee;
using Gee;
using Config;
using Folks;

namespace Unity.PeopleLens {
    const string BUS_NAME = "com.canonical.Unity.Lens.People";
    const string BUS_PATH = "/com/canonical/unity/lens/people";
    const string CONTACT_SCHEME = "contact://";

    /**
    * The Daemon class implements all of the logic for the lens.
    */
    public class Daemon : GLib.Object
    {
		private Unity.Lens lens;
        private Unity.Scope scope;
        private Unity.Scope mock_scope;
        private Folks.IndividualAggregator aggregator;
        // Dee related
        private Dee.SharedModel? streams_model = null;
        private Dee.Model? filtered_model = null;
        private Dee.TextAnalyzer analyzer;
        private Dee.Index index;     
        private Dee.Filter sort_filter;
        public bool streams_ready = false;
        
        private const string CATEGORY_FAVORITES = "Favorites";
        private const string CATEGORY_RECENTS = "Recents";

        private const string ICONS_DIR = Config.LENSESDIR + "/people/";
        private const string ICON_FAVORITE_FILE = Config.PREFIX + "/share/icons/unity-icon-theme/places/svg/group-mostused.svg";

        construct
        {
            /* Set up the lens object (make sure it matches info in the .lens file */
            lens = new Unity.Lens (BUS_PATH, "people");
            lens.search_hint = _("Search people");
            lens.visible = true;
            lens.search_in_global = false;
            lens.filters = null;

            populate_categories ();
            initialize_folks ();

            /* Create a scope so we can add results to the lens */
            scope = new Unity.Scope ("/com/canonical/unity/lens/people/folksscope");
            
            /* Let's ignore searches where only the leading or trailing whitespace
            * changes */
            scope.generate_search_key.connect ((lens_search) =>
            {
                return lens_search.search_string.strip ();
            });

            /* Listen for search changes async */
            scope.search_changed.connect ((lens_search, search_type, cancellable) =>
            { 
                start_search.begin(lens_search, search_type, cancellable);
            });

			scope.preview_uri.connect (preview);

            /* Register the scope we created as local */
            lens.add_local_scope (scope);

            /* Create a mock scope*/
            mock_scope = new Unity.Scope ("/com/canonical/unity/lens/people/mockscope");
            
            mock_scope.generate_search_key.connect ((lens_search) =>
            {
                return lens_search.search_string.strip ();
            });
            
            mock_scope.search_changed.connect ((lens_search, search_type, cancellable) =>
            {
                start_mock_search.begin(lens_search, search_type, cancellable);
            });    

            /* Register the mock scope we created as local */
            lens.add_local_scope (mock_scope);

            streams_model = new Dee.SharedModel ("com.canonical.Friends.Streams");

            if (streams_model is Dee.SharedModel)
                streams_model.notify["synchronized"].connect (on_synchronized_friends_stream);
            
            /* Export the lens on the bus. Unity can see us past this point */
            try {
                lens.export ();
            } catch (IOError error) {
                critical ("Failed to export DBus service for '%s': %s",
                          lens.dbus_path, error.message);
            }
        }

        private void populate_categories ()
        {
            GLib.List<Unity.Category> categories = new GLib.List<Unity.Category> ();
            var icons_dir = File.new_for_path (ICONS_DIR);
            var icon_favorites = File.new_for_path (ICON_FAVORITE_FILE);

            /* The offsets should match those from the Category enum (schemas.vala) */
            var cat = new Unity.Category (_("Favorites"),
                                           new FileIcon (icon_favorites),
                                           Unity.CategoryRenderer.HORIZONTAL_TILE);
            categories.append (cat);

            cat = new Unity.Category (_("Recent Calls"),
                                      new FileIcon (icons_dir.get_child ("people-lens-recent-call.svg")),
                                      Unity.CategoryRenderer.HORIZONTAL_TILE);
            categories.append (cat);

            cat = new Unity.Category (_("A-Z"),
                                      new FileIcon (icons_dir.get_child ("people-lens-a-z.svg")),
                                      Unity.CategoryRenderer.HORIZONTAL_TILE);
            categories.append (cat);

            lens.categories = categories;
        }

        private void initialize_folks ()
        {
            aggregator = new Folks.IndividualAggregator();
            aggregator.prepare();
        }

        private async void start_mock_search (Unity.LensSearch search,
                                              Unity.SearchType search_type,
                                              Cancellable cancellable)
        {
            var model = search.results_model;            
            model.clear ();    

            Gee.ArrayList<HashMap<string, string>> favourites;
            Gee.ArrayList<HashMap<string, string>> recent_calls;
            
            favourites = new Gee.ArrayList<HashMap<string, string>>();
            recent_calls = new Gee.ArrayList<HashMap<string, string>>();

            // Populate favourites
            HashMap<string, string> tom = new HashMap<string, string>();
            tom.set("name", "Tom Dunne");
            tom.set("email", "tom@dunne.com");
            tom.set("phone", "+447967002724");
            tom.set("avatar", ""); // use the social networking mock icon
            tom.set("presence-message", "Rightio then");
            tom.set("remote-source-icon", Config.LENSESDIR + "/people/icon_facebook.png");
            tom.set("remote-post", "I am the one and only ...");
            tom.set("remote-avatar", Config.LENSESDIR + "/people/mocktomfacebookavatar");           
            favourites.add(tom);

            // Populate recent calls
            HashMap<string, string> wendy = new HashMap<string, string>();
            wendy.set("name", "Wendy Smith");
            wendy.set("email", "wendy@smithness.com");
            wendy.set("phone", "+447778668794");
            wendy.set("avatar", "");// use the social networking mock icon
            wendy.set("presence-message", "");
            wendy.set("remote-source-icon", Config.LENSESDIR + "/people/icon_facebook.png");
            wendy.set("remote-post", "heavy night last night, feeling it today...");
            wendy.set("remote-avatar", Config.LENSESDIR + "/people/mockwendyfacebookavatar");                       
            recent_calls.add(wendy);
            
            // Add favourites !
            foreach (var fav in favourites){
                var builder = new VariantBuilder (VariantType.ARRAY);                
		        Variant i;

                foreach(var k in fav.keys){
                    //message ("populating mock with key %s and value %s", k, fav.get(k));
                    i = fav.get(k);
                    builder.add("(sv)", k, i);
                }
                
                Variant output;
				output = new Variant ("a(sv)", builder);

                model.append (CONTACT_SCHEME,
                              fav.get("remote-avatar"),    // populate view friends icon
                              Category.FAVORITES,        // category index
                              "application/contact",     // mime-type of the URI,
                              fav.get("name"),           // display name of the result
                              output.print(true),        // a{sv} - comments, availability. 
                              "");					     // URI for drag and drop
            }

            // Add recent calls !
            foreach (var r in recent_calls){
                var builder = new VariantBuilder (VariantType.ARRAY);                
		        Variant i;

                foreach(var k in r.keys){
                    //message ("populating mock with key %s and value %s", k, r.get(k));
                    i = r.get(k);
                    builder.add("(sv)", k, i);
                }
                Variant output;
				output = new Variant ("a(sv)", builder);

                model.append (CONTACT_SCHEME,
                              r.get("remote-avatar"),   // string representation of an icon
                              Category.RECENT_CALLS,    // category index
                              "application/contact",    // mime-type of the URI,
                              r.get("name"),            // display name of the result
                              output.print(true),       // a{sv} - comments, availability. 
                              "");					    // URI for drag and drop
            }
            search.finished ();
        }
        

        private HashSet<Folks.Individual> folks_search (string search, Cancellable cancellable)
        {
            var results = new HashSet<Folks.Individual>();
            var contacts = aggregator.individuals.values;

            foreach (var contact in contacts) {
                message(".");
                if (cancellable.is_cancelled ()) {
                    return results;
                }

                if (contact.is_user) {
                    continue;
                }

                if (search.length == 0) {
                    results.add(contact);
                    continue;
                }

                // Check name, nickname
                if ((search in contact.nickname.down()) ||
                    (search in contact.alias.down()) ||
                    (search in contact.full_name.down())) {
                    results.add(contact);
                    continue;
                }

                // Check for e-mail
                foreach(var email in contact.email_addresses) {
                    if (search in email.value.down()) {
                        results.add(contact);
                        continue;
                    }
                }

                // Check for phone number
                foreach(var phone in contact.phone_numbers) {
                    if (search in phone.value.down()) {
                        results.add(contact);
                        continue;
                    }
                }

                // Check for IM
                foreach(var im in contact.im_addresses.get_values()) {
                    if (search in im.value.down()) {
                        results.add(contact);
                        continue;
                    }
                }

                // Check for Address
                foreach(var addressDetails in contact.postal_addresses) {
                    var address = addressDetails.value;
                    if (search in address.country.down() ||
                        search in address.extension.down() ||
                        search in address.locality.down() ||
                        search in address.po_box.down() ||
                        search in address.postal_code.down() ||
                        search in address.region.down() ||
                        search in address.street.down()) {
                        results.add(contact);
                        continue;
                    }
                }
            }
            return results;
        }

        private async void start_search (Unity.LensSearch search,
                                         Unity.SearchType search_type,
                                         Cancellable cancellable)
        {
            var model = search.results_model;
            var results = folks_search (search.search_string.down(), cancellable);
            var cache = Folks.AvatarCache.dup ();

            model.clear ();

            foreach (var folk in results) {
                message(":");
                
                if (cancellable.is_cancelled ()) {
                    /* we got another search meanwhile, let's just end this one */
                    search.finished ();
                    return;
                }
              
                var avatar = "";
                
                if (folk.avatar != null) {
                    cache.store_avatar(folk.id, folk.avatar);
                    var avatarUri = cache.build_uri_for_avatar(folk.id);
                    var file = File.new_for_uri (avatarUri);
                    avatar = file.get_path();
                }

                var category = Category.ALL;
                if (CATEGORY_RECENTS in folk.groups) {
                    category = Category.RECENT_CALLS;
                } else if (folk.is_favourite ||
                    (CATEGORY_FAVORITES in folk.groups)) {
                    category = Category.FAVORITES;
                }

                var displayName = _("unknown");
                if (folk.nickname.length > 0) {
                    displayName = folk.nickname;
                } else if (folk.full_name.length > 0) {
                    displayName = folk.full_name;
                } else if (folk.alias.length > 0) {
                    displayName = folk.alias;
                }
				
                var builder = new VariantBuilder (VariantType.ARRAY);                
                
                // Presence_status
                var status = "offline";
                if (folk.is_online()) {
                    status = Folks.PresenceDetails.get_default_message_from_type (folk.presence_type);
                }
                Variant i = status;
                builder.add ("(sv)", "presence-status", i);

                // Presence message
		        if (folk.presence_message != null && folk.presence_message.char_count() > 0) {
					i = folk.presence_message;						
					builder.add ("(sv)", "presence-message", i);
		        }

                var email = "";
                if (folk.email_addresses.size > 0) {
                    var email_address = folk.email_addresses.to_array()[0].value;
                    email += "\n" + email_address;
		            i=email_address;
		            builder.add("(sv)", "email", i);
                }
				
                if (folk.phone_numbers.size > 0) {
                    var phone_number = folk.phone_numbers.to_array()[0].value;
                    i=phone_number;
                    builder.add("(sv)", "phone", i);
                }


                if (this.streams_ready){

                    HashMap<string, string>? post;
                    post = fetch_post(folk);

                    if (post != null){
                        message ("Chosen %s post from %s = %s @ %s", post.get("service"), folk.full_name, post.get("post"), post.get("time"));
                        // social-post-origin
                        i = Config.LENSESDIR + "/people/icon_%s.png".printf(post.get("service"));
                        builder.add("(sv)", "remote-source-icon", i);
                        // social-post
                        i = post.get("post");
                        builder.add("(sv)", "remote-post", i);
                        // social-post-icon
                        i = post.get("icon_path");                   
                        builder.add("(sv)", "remote-avatar", i);
                        // social post uri
                        i = post.get("post_uri");
                        builder.add("(sv)", "remote-post-uri", i);
                        if (avatar.char_count() == 0 && post.get("icon_path").char_count() > 0){
                            avatar = post.get("icon_path"); // TODO - do you need to dup here
                        }
                    }
                }
                // Just in case we didn't manage to set the avatar from either the folk or friends (hopefully very rare)
                if (avatar.char_count() == 0)
                    avatar = ICONS_DIR + "/people-lens-default.png";
                    
                Variant output;
				output = new Variant ("a(sv)", builder);		
				
                model.append (CONTACT_SCHEME + folk.id, // URI of the result
                              avatar,                   // string representation of an icon
                              category,                 // category index
                              "application/contact",    // mime-type of the URI,
                              displayName,              // display name of the result
                              output.print(true),       // a{sv} - comments, availability. 
                              "");					    // URI for drag and drop
            }
            search.finished ();
        }

        private HashMap<string, string>? fetch_post(Folks.Individual folk)
        {
            int facebook_id = 0;
            int twitter_id = 0;

            Gee.HashMap<string, string>? facebook_results = null;
            Gee.HashMap<string, string>? twitter_results = null;

            facebook_id = extract_facebook_id(folk);
            twitter_id = extract_twitter_id(folk);

            if (twitter_id != 0)
                twitter_results = extract_post_from_service (folk, "twitter", twitter_id);
            if (facebook_id != 0)
                facebook_results = extract_post_from_service (folk, "facebook", facebook_id);

            bool use_twitter = twitter_results == null ? false : true;
            bool use_facebook = facebook_results == null ? false : true;

            if (use_twitter & use_facebook)
                // the one with the most recent time_stamp
                return determine_most_recent_post (twitter_results, facebook_results);
            if (use_twitter & !use_facebook)
                return twitter_results;
            if (use_facebook & !use_twitter)
                return facebook_results;
            return null;
        }

        // Why on earth doesn't Vala have a proper Time API like any other decent language.
        private HashMap<string, string> determine_most_recent_post (HashMap<string, string> twitter_results, HashMap<string, string> facebook_results)
        {
            var twitter_time = twitter_results.get("time").split("-");
            var facebook_time = facebook_results.get("time").split("-");

            // test years
            if (twitter_time[0] != facebook_time[0]){
                return int.parse(twitter_time[0]) > int.parse(facebook_time[0]) ? twitter_results : facebook_results;
            }
            // test months
            if (twitter_time[1] != facebook_time[1]){
                return int.parse(twitter_time[1]) > int.parse(facebook_time[1]) ? twitter_results : facebook_results;
            }
            // test days
            var twitter_day  = twitter_time[2].split("T")[0];
            var facebook_day  = facebook_time[2].split("T")[0];

            if(twitter_day != facebook_day){
                return int.parse(twitter_day) > int.parse(facebook_day) ? twitter_results : facebook_results;                 
            }

            // test hour
            var twitter_hour  = twitter_time[2].split("T")[1].split(":")[0];
            var facebook_hour  = facebook_time[2].split("T")[1].split(":")[0];

            if(twitter_hour != facebook_hour){
                return int.parse(twitter_hour) > int.parse(facebook_hour) ? twitter_results : facebook_results;                 
            }
            // just go with twitter - usually more informative than facebook garbage.
            return twitter_results; 
        }

        private HashMap<string, string>? extract_post_from_service (Folks.Individual folk, string service, int service_id)
        {
            Dee.ResultSet result_set;
            result_set = this.index.lookup(service_id.to_string(), Dee.TermMatchFlag.EXACT);
            
            unowned string? post = null;
            unowned string? time = null;
            unowned string? icon_local_path = null;
            unowned string? post_uri = null;

            message ("ResultSet for %s for %s is %u in length", service, folk.full_name, result_set.get_n_rows());
            
            foreach (var result in result_set) {
                
                string account_service = "";                         
                var message_ids = filtered_model.get_value(result, Friends.StreamModelColumn.MESSAGE_IDS);
                bool valid = false;
                
                foreach (var message_id in message_ids)
                {
                    // There is a possibility with the search above that the id might match with another service.
                    var message_id_item = (string[])message_id;
                    account_service = message_id_item[0];
                    valid = message_id_item[0] == service;
                    if(valid)
                        break;
                }

                if (!valid)
                    continue;

                post = filtered_model.get_string (result, Friends.StreamModelColumn.MESSAGE);          

                // If the post has no chars ignore it (probably a pic or a link) or its a reply to something ignore, get the next genuine post ...
                if (post.char_count() == 0 || post.has_prefix ("reply_to"))
                    continue;

                time = filtered_model.get_string (result, Friends.StreamModelColumn.TIMESTAMP);
                icon_local_path = filtered_model.get_string (result, Friends.StreamModelColumn.ICON_URI);
                post = filtered_model.get_string (result, Friends.StreamModelColumn.MESSAGE);          
				post_uri = filtered_model.get_string (result, Friends.StreamModelColumn.URL);          
                message ("%s - found a %s post %s for this contact %s", time, service, post, folk.full_name);
                break;
            }
            // in case it matches an id not from that service, the loop may complete (and not return) without setting any of the strings.
            if(time == null)
                return null;

            Gee.HashMap<string, string> results = new Gee.HashMap<string, string>();
            results.set("service", service);
            results.set("post", post);
            results.set("time", time);
            results.set("icon_path", icon_local_path); 
            results.set("post_uri",  post_uri); 
            return results;
        }

        private int extract_facebook_id (Folks.Individual folk)
        {
            var facebook_id = 0;
            if (folk.web_service_addresses.get_keys() != null){
                foreach (var k in folk.web_service_addresses.get_keys()){
                    if (k == "jabber"){
                        foreach (var wsfd in folk.web_service_addresses.get(k)){
                            if(wsfd.value.contains ("facebook")){
                                var parts = wsfd.value.split("@");
                                //message ("Found a facebook id id %i for %s", int.parse(parts[0]).abs(), folk.full_name);
                                facebook_id = int.parse(parts[0]).abs();
                                break; 
                            }
                        }
                    }
                }
            }
            return facebook_id;
        }

        private int extract_twitter_id (Folks.Individual folk)
        {
            var twitter_id = 0;
            if (folk.web_service_addresses.get_keys() != null){
                foreach (var k in folk.web_service_addresses.get_keys()){
                    if (k == "twitter-id"){
                        foreach (var wsfd in folk.web_service_addresses.get(k)){
                            //message ("Found a twitterer id %i for %s", int.parse(wsfd.value).abs(), folk.full_name);
                            twitter_id = int.parse(wsfd.value).abs();
                            break; 
                        }
                    }
                }
            }
            return twitter_id;
        }

        private void on_synchronized_friends_stream ()
        {
            debug ("on_synchronized_friends_stream");

            if (!streams_model.is_synchronized() || streams_model.is_leader())
                return;
            
            sort_filter = Dee.Filter.new_collator (Friends.StreamModelColumn.TIMESTAMP); 
            filtered_model = new Dee.FilterModel (streams_model, sort_filter);
            analyzer = new Dee.TextAnalyzer ();

            var reader = Dee.ModelReader.new ((filtered_model, iter) =>
            {
                var service_col = Friends.StreamModelColumn.MESSAGE_IDS;
                var sender_id_col = Friends.StreamModelColumn.SENDER_ID;
                var message_col = Friends.StreamModelColumn.MESSAGE;

                string[] services = {}; 
                
                var message_ids = filtered_model.get_value(iter, service_col);
                
                foreach (var message_id in message_ids){
                    var message_id_item = (string[])message_id;
                    services += message_id_item[0];
                }
                
                var sender_id = filtered_model.get_value(iter,sender_id_col);
                var message = filtered_model.get_value(iter, message_col);
                var service = string.joinv(",", services);       

                debug ("service = %s", service);
                debug ("TYPE of sender id col = %s", sender_id.get_type_string());
                debug ("TYPE of message col = %s", message.get_type_string());
                
                return "%s\n%s\n%s".printf (service,
                                            filtered_model.get_string (iter, sender_id_col),
                                            filtered_model.get_string (iter, message_col));
            });
            index = new Dee.TreeIndex (filtered_model, analyzer, reader);
            streams_model.notify["row-added"].connect ((source) => {
                message ("ROW ADDED");
            });
            this.streams_ready = true;
        }
	public Unity.Preview preview (string uri)
        {
    	    debug ("Previewing: %s", uri);
            Unity.PeoplePreview preview = null;
            var contacts = aggregator.individuals.values;
            var cache = Folks.AvatarCache.dup ();

    	    foreach (var folk in contacts)
            {
    		var url = CONTACT_SCHEME + folk.id;
    		if (uri == url)
    		{
    		    debug ("Found %s", url);
		    Icon? avatar = null;
                
                    if (folk.avatar != null) {
                        var avatarUri = cache.build_uri_for_avatar(folk.id);
                        var file = File.new_for_uri (avatarUri);
                        avatar = new FileIcon(file);
                    }

		    var post = fetch_post(folk);

		    if (avatar == null && post != null && post.get("icon_path").char_count() > 0) {
                        var avatarUri = post.get("icon_path");
		        var file = File.new_for_uri (avatarUri);
                        avatar = new FileIcon(file);
                    }

                    if (avatar == null)
		    {
                        var avatarUri = ICONS_DIR + "/people-lens-default.png";
		        var file = File.new_for_uri (avatarUri);
                        avatar = new FileIcon(file);
		    }

                    var displayName = _("unknown");
                    if (folk.nickname.length > 0) {
                        displayName = folk.nickname;
                    } else if (folk.full_name.length > 0) {
                        displayName = folk.full_name;
                    } else if (folk.alias.length > 0) {
                        displayName = folk.alias;
                    }

    	            preview = new Unity.PeoplePreview(displayName, avatar);

		    foreach(var email in folk.email_addresses) {
		        var type_ = "other";
			if (email.parameters.contains("type")) {
			    type_ = "";
			    for (var i=0; i<email.parameters.get("type").to_array().length; i++) {
			        var type__ = email.parameters.get("type").to_array()[i];
				if (type_ == "") {
				    type_ = type__;
                                } else {
				    type_ = type_ + " " + type__;
                                }
                            }
			}
                        preview.add_email(new Unity.PeoplePreview.Email (type_,email.value.down()));
                    }

                    foreach(var phone in folk.phone_numbers) {
		        var type_ = "";
			var location = "";
			if (phone.parameters.contains("type")) {
			    for (var i=0; i<phone.parameters.get("type").to_array().length; i++) {
			        var type__ = phone.parameters.get("type").to_array()[i];
				if (type__ == "voice" || type__ == "fax" || type__ == "pager" || type__ == "cell") {
				    type_ = type__;
				} else if (type__ == "home" || type__ == "work") {
				    location = type__;
                                } else {
				    debug ("Unknown phone type: %s", type__);
				}
                            }
			}
                        preview.add_phone(new Unity.PeoplePreview.Phone (type_, location, phone.value.down()));
                    }

                    foreach(var im in folk.im_addresses.get_keys()) {
		        var address = folk.im_addresses.get(im).to_array()[0].value;
			preview.add_imaccount(new Unity.PeoplePreview.IMAccount(im, address));
                    }

                    foreach(var addressDetails in folk.postal_addresses) {
                        var address = addressDetails.value.to_string();
			var type_ = "Other";
			if (addressDetails.parameters.contains("type")) {
			    type_ = "";
			    for (var i=0; i<addressDetails.parameters.get("type").to_array().length; i++) {
			        var type__ = addressDetails.parameters.get("type").to_array()[i];
				if (type_ == "") {
				    type_ = type__;
                                } else {
				    type_ = type_ + " " + type__;
                                }
                            }
			} else if (addressDetails.id != null) {
			   type_ = addressDetails.id;
			}
                        preview.add_address(new Unity.PeoplePreview.Address (type_, address, addressDetails.value.po_box, addressDetails.value.extension, addressDetails.value.street, addressDetails.value.locality, addressDetails.value.region, addressDetails.value.postal_code, addressDetails.value.country, addressDetails.value.address_format));
                    }

		    if (post != null) {
		       preview.status = post.get("post");
		       preview.status_time = post.get("time");
		       preview.status_service = post.get("service");
		       var status_service_icon_uri = Config.LENSESDIR + "/people/icon_%s.png".printf(preview.status_service);
		       var status_service_icon_file = File.new_for_uri (status_service_icon_uri);
		       preview.status_service_icon = new FileIcon(status_service_icon_file);
		       preview.status_post_uri = post.get("post_uri");
		    }

		    preview.favorite = folk.is_favourite;

    		    break;
                }
            }
    	    return preview;
        }
    } 
} 
